package javax.imageio.stream;

import java.io.UTFDataFormatException;
import java.nio.ByteOrder;
import java.io.IOException;

public abstract class ImageOutputStreamImpl extends ImageInputStreamImpl implements ImageOutputStream
{
    public ImageOutputStreamImpl() {
    }
    
    @Override
    public abstract void write(int p0) throws IOException;
    
    @Override
    public void write(byte[] array) throws IOException {
        this.write(array, 0, array.length);
    }
    
    @Override
    public abstract void write(byte[] p0, int p1, int p2) throws IOException;
    
    @Override
    public void writeBoolean(boolean b) throws IOException {
        this.write(b ? 1 : 0);
    }
    
    @Override
    public void writeByte(int n) throws IOException {
        this.write(n);
    }
    
    @Override
    public void writeShort(int n) throws IOException {
        if (this.byteOrder == ByteOrder.BIG_ENDIAN) {
            this.byteBuf[0] = (byte)(n >>> 8);
            this.byteBuf[1] = (byte)(n >>> 0);
        }
        else {
            this.byteBuf[0] = (byte)(n >>> 0);
            this.byteBuf[1] = (byte)(n >>> 8);
        }
        this.write(this.byteBuf, 0, 2);
    }
    
    @Override
    public void writeChar(int n) throws IOException {
        this.writeShort(n);
    }
    
    @Override
    public void writeInt(int n) throws IOException {
        if (this.byteOrder == ByteOrder.BIG_ENDIAN) {
            this.byteBuf[0] = (byte)(n >>> 24);
            this.byteBuf[1] = (byte)(n >>> 16);
            this.byteBuf[2] = (byte)(n >>> 8);
            this.byteBuf[3] = (byte)(n >>> 0);
        }
        else {
            this.byteBuf[0] = (byte)(n >>> 0);
            this.byteBuf[1] = (byte)(n >>> 8);
            this.byteBuf[2] = (byte)(n >>> 16);
            this.byteBuf[3] = (byte)(n >>> 24);
        }
        this.write(this.byteBuf, 0, 4);
    }
    
    @Override
    public void writeLong(long n) throws IOException {
        if (this.byteOrder == ByteOrder.BIG_ENDIAN) {
            this.byteBuf[0] = (byte)(int)(n >>> 56);
            this.byteBuf[1] = (byte)(int)(n >>> 48);
            this.byteBuf[2] = (byte)(int)(n >>> 40);
            this.byteBuf[3] = (byte)(int)(n >>> 32);
            this.byteBuf[4] = (byte)(int)(n >>> 24);
            this.byteBuf[5] = (byte)(int)(n >>> 16);
            this.byteBuf[6] = (byte)(int)(n >>> 8);
            this.byteBuf[7] = (byte)(int)(n >>> 0);
        }
        else {
            this.byteBuf[0] = (byte)(int)(n >>> 0);
            this.byteBuf[1] = (byte)(int)(n >>> 8);
            this.byteBuf[2] = (byte)(int)(n >>> 16);
            this.byteBuf[3] = (byte)(int)(n >>> 24);
            this.byteBuf[4] = (byte)(int)(n >>> 32);
            this.byteBuf[5] = (byte)(int)(n >>> 40);
            this.byteBuf[6] = (byte)(int)(n >>> 48);
            this.byteBuf[7] = (byte)(int)(n >>> 56);
        }
        this.write(this.byteBuf, 0, 4);
        this.write(this.byteBuf, 4, 4);
    }
    
    @Override
    public void writeFloat(float value) throws IOException {
        this.writeInt(Float.floatToIntBits(value));
    }
    
    @Override
    public void writeDouble(double value) throws IOException {
        this.writeLong(Double.doubleToLongBits(value));
    }
    
    @Override
    public void writeBytes(String s) throws IOException {
        for (int length = s.length(), i = 0; i < length; ++i) {
            this.write((int)(byte)s.charAt(i));
        }
    }
    
    @Override
    public void writeChars(String s) throws IOException {
        int length = s.length();
        byte[] array = new byte[length * 2];
        int n = 0;
        if (this.byteOrder == ByteOrder.BIG_ENDIAN) {
            for (int j = 0; j < length; ++j) {
                int char2 = s.charAt(j);
                array[n++] = (byte)(char2 >>> 8);
                array[n++] = (byte)(char2 >>> 0);
            }
        }
        else {
            for (int i = 0; i < length; ++i) {
                int char1 = s.charAt(i);
                array[n++] = (byte)(char1 >>> 0);
                array[n++] = (byte)(char1 >>> 8);
            }
        }
        this.write(array, 0, length * 2);
    }
    
    @Override
    public void writeUTF(String s) throws IOException {
        int length = s.length();
        int n = 0;
        char[] dst = new char[length];
        int n2 = 0;
        s.getChars(0, length, dst, 0);
        for (int n3 : dst) {
            if (n3 >= '\u0001' && n3 <= '\u007f') {
                ++n;
            }
            else if (n3 > '߿') {
                n += 3;
            }
            else {
                n += 2;
            }
        }
        if (n > 65535) {
            throw new UTFDataFormatException("utflen > 65536!");
        }
        byte[] array = new byte[n + 2];
        array[n2++] = (byte)(n >>> 8 & 0xFF);
        array[n2++] = (byte)(n >>> 0 & 0xFF);
        for (int n4 : dst) {
            if (n4 >= '\u0001' && n4 <= '\u007f') {
                array[n2++] = (byte)n4;
            }
            else if (n4 > '߿') {
                array[n2++] = (byte)(0xE0 | (n4 >> 12 & 0xF));
                array[n2++] = (byte)(0x80 | (n4 >> 6 & 0x3F));
                array[n2++] = (byte)(0x80 | (n4 >> 0 & 0x3F));
            }
            else {
                array[n2++] = (byte)(0xC0 | (n4 >> 6 & 0x1F));
                array[n2++] = (byte)(0x80 | (n4 >> 0 & 0x3F));
            }
        }
        this.write(array, 0, n + 2);
        return;
    }
    
    @Override
    public void writeShorts(short[] array, int n, int n2) throws IOException {
        if (n < 0 || n2 < 0 || n + n2 > array.length || n + n2 < 0) {
            throw new IndexOutOfBoundsException("off < 0 || len < 0 || off + len > s.length!");
        }
        byte[] array2 = new byte[n2 * 2];
        int n3 = 0;
        if (this.byteOrder == ByteOrder.BIG_ENDIAN) {
            for (int j = 0; j < n2; ++j) {
                int n5 = array[n + j];
                array2[n3++] = (byte)(n5 >>> 8);
                array2[n3++] = (byte)(n5 >>> 0);
            }
        }
        else {
            for (int i = 0; i < n2; ++i) {
                int n4 = array[n + i];
                array2[n3++] = (byte)(n4 >>> 0);
                array2[n3++] = (byte)(n4 >>> 8);
            }
        }
        this.write(array2, 0, n2 * 2);
        return;
    }
    
    @Override
    public void writeChars(char[] array, int n, int n2) throws IOException {
        if (n < 0 || n2 < 0 || n + n2 > array.length || n + n2 < 0) {
            throw new IndexOutOfBoundsException("off < 0 || len < 0 || off + len > c.length!");
        }
        byte[] array2 = new byte[n2 * 2];
        int n3 = 0;
        if (this.byteOrder == ByteOrder.BIG_ENDIAN) {
            for (int j = 0; j < n2; ++j) {
                int n5 = array[n + j];
                array2[n3++] = (byte)(n5 >>> 8);
                array2[n3++] = (byte)(n5 >>> 0);
            }
        }
        else {
            for (int i = 0; i < n2; ++i) {
                int n4 = array[n + i];
                array2[n3++] = (byte)(n4 >>> 0);
                array2[n3++] = (byte)(n4 >>> 8);
            }
        }
        this.write(array2, 0, n2 * 2);
        return;
    }
    
    @Override
    public void writeInts(int[] array, int n, int n2) throws IOException {
        if (n < 0 || n2 < 0 || n + n2 > array.length || n + n2 < 0) {
            throw new IndexOutOfBoundsException("off < 0 || len < 0 || off + len > i.length!");
        }
        byte[] array2 = new byte[n2 * 4];
        int n3 = 0;
        if (this.byteOrder == ByteOrder.BIG_ENDIAN) {
            for (int j = 0; j < n2; ++j) {
                int n5 = array[n + j];
                array2[n3++] = (byte)(n5 >>> 24);
                array2[n3++] = (byte)(n5 >>> 16);
                array2[n3++] = (byte)(n5 >>> 8);
                array2[n3++] = (byte)(n5 >>> 0);
            }
        }
        else {
            for (int i = 0; i < n2; ++i) {
                int n4 = array[n + i];
                array2[n3++] = (byte)(n4 >>> 0);
                array2[n3++] = (byte)(n4 >>> 8);
                array2[n3++] = (byte)(n4 >>> 16);
                array2[n3++] = (byte)(n4 >>> 24);
            }
        }
        this.write(array2, 0, n2 * 4);
        return;
    }
    
    @Override
    public void writeLongs(long[] array, int n, int n2) throws IOException {
        if (n < 0 || n2 < 0 || n + n2 > array.length || n + n2 < 0) {
            throw new IndexOutOfBoundsException("off < 0 || len < 0 || off + len > l.length!");
        }
        byte[] array2 = new byte[n2 * 8];
        int n3 = 0;
        if (this.byteOrder == ByteOrder.BIG_ENDIAN) {
            for (int j = 0; j < n2; ++j) {
                long n5 = array[n + j];
                array2[n3++] = (byte)(int)(n5 >>> 56);
                array2[n3++] = (byte)(int)(n5 >>> 48);
                array2[n3++] = (byte)(int)(n5 >>> 40);
                array2[n3++] = (byte)(int)(n5 >>> 32);
                array2[n3++] = (byte)(int)(n5 >>> 24);
                array2[n3++] = (byte)(int)(n5 >>> 16);
                array2[n3++] = (byte)(int)(n5 >>> 8);
                array2[n3++] = (byte)(int)(n5 >>> 0);
            }
        }
        else {
            for (int i = 0; i < n2; ++i) {
                long n4 = array[n + i];
                array2[n3++] = (byte)(int)(n4 >>> 0);
                array2[n3++] = (byte)(int)(n4 >>> 8);
                array2[n3++] = (byte)(int)(n4 >>> 16);
                array2[n3++] = (byte)(int)(n4 >>> 24);
                array2[n3++] = (byte)(int)(n4 >>> 32);
                array2[n3++] = (byte)(int)(n4 >>> 40);
                array2[n3++] = (byte)(int)(n4 >>> 48);
                array2[n3++] = (byte)(int)(n4 >>> 56);
            }
        }
        this.write(array2, 0, n2 * 8);
        return;
    }
    
    @Override
    public void writeFloats(float[] array, int n, int n2) throws IOException {
        if (n < 0 || n2 < 0 || n + n2 > array.length || n + n2 < 0) {
            throw new IndexOutOfBoundsException("off < 0 || len < 0 || off + len > f.length!");
        }
        byte[] array2 = new byte[n2 * 4];
        int n3 = 0;
        if (this.byteOrder == ByteOrder.BIG_ENDIAN) {
            for (int j = 0; j < n2; ++j) {
                int floatToIntBits2 = Float.floatToIntBits(array[n + j]);
                array2[n3++] = (byte)(floatToIntBits2 >>> 24);
                array2[n3++] = (byte)(floatToIntBits2 >>> 16);
                array2[n3++] = (byte)(floatToIntBits2 >>> 8);
                array2[n3++] = (byte)(floatToIntBits2 >>> 0);
            }
        }
        else {
            for (int i = 0; i < n2; ++i) {
                int floatToIntBits = Float.floatToIntBits(array[n + i]);
                array2[n3++] = (byte)(floatToIntBits >>> 0);
                array2[n3++] = (byte)(floatToIntBits >>> 8);
                array2[n3++] = (byte)(floatToIntBits >>> 16);
                array2[n3++] = (byte)(floatToIntBits >>> 24);
            }
        }
        this.write(array2, 0, n2 * 4);
        return;
    }
    
    @Override
    public void writeDoubles(double[] array, int n, int n2) throws IOException {
        if (n < 0 || n2 < 0 || n + n2 > array.length || n + n2 < 0) {
            throw new IndexOutOfBoundsException("off < 0 || len < 0 || off + len > d.length!");
        }
        byte[] array2 = new byte[n2 * 8];
        int n3 = 0;
        if (this.byteOrder == ByteOrder.BIG_ENDIAN) {
            for (int j = 0; j < n2; ++j) {
                long doubleToLongBits2 = Double.doubleToLongBits(array[n + j]);
                array2[n3++] = (byte)(int)(doubleToLongBits2 >>> 56);
                array2[n3++] = (byte)(int)(doubleToLongBits2 >>> 48);
                array2[n3++] = (byte)(int)(doubleToLongBits2 >>> 40);
                array2[n3++] = (byte)(int)(doubleToLongBits2 >>> 32);
                array2[n3++] = (byte)(int)(doubleToLongBits2 >>> 24);
                array2[n3++] = (byte)(int)(doubleToLongBits2 >>> 16);
                array2[n3++] = (byte)(int)(doubleToLongBits2 >>> 8);
                array2[n3++] = (byte)(int)(doubleToLongBits2 >>> 0);
            }
        }
        else {
            for (int i = 0; i < n2; ++i) {
                long doubleToLongBits = Double.doubleToLongBits(array[n + i]);
                array2[n3++] = (byte)(int)(doubleToLongBits >>> 0);
                array2[n3++] = (byte)(int)(doubleToLongBits >>> 8);
                array2[n3++] = (byte)(int)(doubleToLongBits >>> 16);
                array2[n3++] = (byte)(int)(doubleToLongBits >>> 24);
                array2[n3++] = (byte)(int)(doubleToLongBits >>> 32);
                array2[n3++] = (byte)(int)(doubleToLongBits >>> 40);
                array2[n3++] = (byte)(int)(doubleToLongBits >>> 48);
                array2[n3++] = (byte)(int)(doubleToLongBits >>> 56);
            }
        }
        this.write(array2, 0, n2 * 8);
        return;
    }
    
    @Override
    public void writeBit(int n) throws IOException {
        this.writeBits(0x1L & (long)n, 1);
    }
    
    @Override
    public void writeBits(long n, int bitOffset) throws IOException {
        this.checkClosed();
        if (bitOffset < 0 || bitOffset > 64) {
            throw new IllegalArgumentException("Bad value for numBits!");
        }
        if (bitOffset == 0) {
            return;
        }
        if (this.getStreamPosition() > 0L || this.bitOffset > 0) {
            int bitOffset2 = this.bitOffset;
            int read = this.read();
            if (read != -1) {
                this.seek(this.getStreamPosition() - 1L);
            }
            else {
                read = 0;
            }
            if (bitOffset + bitOffset2 < 8) {
                int n4 = 8 - (bitOffset2 + bitOffset);
                int n5 = -1 >>> 32 - bitOffset;
                this.write((int)((long)(read & ~(n5 << n4)) | (n & (long)n5) << n4));
                this.seek(this.getStreamPosition() - 1L);
                this.bitOffset = bitOffset2 + bitOffset;
                bitOffset = 0;
            }
            else {
                int n2 = 8 - bitOffset2;
                int n3 = -1 >>> 32 - n2;
                this.write((int)((long)(read & ~n3) | (n >> bitOffset - n2 & (long)n3)));
                bitOffset -= n2;
            }
        }
        if (bitOffset > 7) {
            int n6 = bitOffset % 8;
            for (int i = bitOffset / 8; i > 0; --i) {
                int n7 = (i - 1) * 8 + n6;
                this.write((int)((n7 == 0) ? (n & 0xFFL) : (n >> n7 & 0xFFL)));
            }
            bitOffset = n6;
        }
        if (bitOffset != 0) {
            int read2 = this.read();
            if (read2 != -1) {
                this.seek(this.getStreamPosition() - 1L);
            }
            else {
                read2 = 0;
            }
            int n8 = 8 - bitOffset;
            int n9 = -1 >>> 32 - bitOffset;
            this.write((int)((long)(read2 & ~(n9 << n8)) | (n & (long)n9) << n8));
            this.seek(this.getStreamPosition() - 1L);
            this.bitOffset = bitOffset;
        }
        return;
    }
    
    protected final void flushBits() throws IOException {
        this.checkClosed();
        if (this.bitOffset != 0) {
            int bitOffset = this.bitOffset;
            int read = this.read();
            int n;
            if (read < 0) {
                n = 0;
                this.bitOffset = 0;
            }
            else {
                this.seek(this.getStreamPosition() - 1L);
                n = (read & -1 << 8 - bitOffset);
            }
            this.write(n);
        }
    }
}
